@@ -15,6 +15,7 @@
 
 #include "xtl/xsequence.hpp"
 
+#include "xbuffer_adaptor.hpp"
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
@@ -137,7 +138,7 @@ namespace xt
         using inner_shape_type = shape_type;
         using inner_strides_type = strides_type;
         using inner_backstrides_type = backstrides_type;
-        using temporary_type = xarray_container<container_type, L, SC, Tag>;
+        using temporary_type = xarray_container<temporary_container_t<container_type>, L, SC, Tag>;
         static constexpr layout_type layout = L;
     };
 
