@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XBROADCAST_HPP
-#define XBROADCAST_HPP
+#ifndef XTENSOR_BROADCAST_HPP
+#define XTENSOR_BROADCAST_HPP
 
 #include <algorithm>
 #include <array>
@@ -17,6 +17,8 @@
 #include <type_traits>
 #include <utility>
 
+#include "xtl/xsequence.hpp"
+
 #include "xexpression.hpp"
 #include "xiterable.hpp"
 #include "xstrides.hpp"
@@ -54,10 +56,6 @@ namespace xt
         using inner_shape_type = promote_shape_t<typename xexpression_type::shape_type, X>;
         using const_stepper = typename xexpression_type::const_stepper;
         using stepper = const_stepper;
-        using const_iterator = xiterator<const_stepper, inner_shape_type*, DEFAULT_LAYOUT>;
-        using iterator = const_iterator;
-        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
-        using reverse_iterator = std::reverse_iterator<iterator>;
     };
 
     /**
@@ -75,10 +73,10 @@ namespace xt
      */
     template <class CT, class X>
     class xbroadcast : public xexpression<xbroadcast<CT, X>>,
-                       public xexpression_const_iterable<xbroadcast<CT, X>>
+                       public xconst_iterable<xbroadcast<CT, X>>
     {
-
     public:
+
         using self_type = xbroadcast<CT, X>;
         using xexpression_type = std::decay_t<CT>;
 
@@ -90,7 +88,7 @@ namespace xt
         using size_type = typename xexpression_type::size_type;
         using difference_type = typename xexpression_type::difference_type;
 
-        using iterable_base = xexpression_const_iterable<self_type>;
+        using iterable_base = xconst_iterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = inner_shape_type;
 
@@ -98,7 +96,8 @@ namespace xt
         using const_stepper = typename iterable_base::const_stepper;
 
         static constexpr layout_type static_layout = xexpression_type::static_layout;
-        static constexpr bool contiguous_layout = xexpression_type::contiguous_layout;
+        //static constexpr bool contiguous_layout = xexpression_type::contiguous_layout;
+        static constexpr bool contiguous_layout = false;
 
         template <class CTA, class S>
         xbroadcast(CTA&& e, S&& s) noexcept;
@@ -110,7 +109,14 @@ namespace xt
 
         template <class... Args>
         const_reference operator()(Args... args) const;
-        const_reference operator[](const xindex& index) const;
+
+        template <class... Args>
+        const_reference at(Args... args) const;
+
+        template <class S>
+        disable_integral_t<S, const_reference> operator[](const S& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -128,6 +134,7 @@ namespace xt
         const_stepper stepper_end(const S& shape, layout_type l) const noexcept;
 
     private:
+
         CT m_e;
         inner_shape_type m_shape;
     };
@@ -151,7 +158,7 @@ namespace xt
     {
         using broadcast_type = xbroadcast<const_xclosure_t<E>, S>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), xtl::forward_sequence<shape_type>(s));
     }
 
 #ifdef X_OLD_CLANG
@@ -160,7 +167,7 @@ namespace xt
     {
         using broadcast_type = xbroadcast<const_xclosure_t<E>, std::vector<std::size_t>>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), xtl::forward_sequence<shape_type>(s));
     }
 #else
     template <class E, class I, std::size_t L>
@@ -168,7 +175,7 @@ namespace xt
     {
         using broadcast_type = xbroadcast<const_xclosure_t<E>, std::array<std::size_t, L>>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), xtl::forward_sequence<shape_type>(s));
     }
 #endif
 
@@ -252,6 +259,23 @@ namespace xt
         return detail::get_element(m_e, args...);
     }
 
+    /**
+     * Returns a constant reference to the element at the specified position in the expression,
+     * after dimension and bounds checking.
+     * @param args a list of indices specifying the position in the function. Indices
+     * must be unsigned integers, the number of indices should be equal to the number of dimensions
+     * of the expression.
+     * @exception std::out_of_range if the number of argument is greater than the number of dimensions
+     * or if indices are out of bounds.
+     */
+    template <class CT, class X>
+    template <class... Args>
+    inline auto xbroadcast<CT, X>::at(Args... args) const -> const_reference
+    {
+        check_access(shape(), static_cast<size_type>(args)...);
+        return this->operator()(args...);
+    }
+
     /**
      * Returns a constant reference to the element at the specified position in the expression.
      * @param index a sequence of indices specifying the position in the function. Indices
@@ -259,11 +283,20 @@ namespace xt
      * than the number of dimensions of the container.
      */
     template <class CT, class X>
-    inline auto xbroadcast<CT, X>::operator[](const xindex& index) const -> const_reference
+    template <class S>
+    inline auto xbroadcast<CT, X>::operator[](const S& index) const
+        -> disable_integral_t<S, const_reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class CT, class X>
+    template <class I>
+    inline auto xbroadcast<CT, X>::operator[](std::initializer_list<I> index) const -> const_reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class CT, class X>
     inline auto xbroadcast<CT, X>::operator[](size_type i) const -> const_reference
     {
@@ -274,7 +307,7 @@ namespace xt
      * Returns a constant reference to the element at the specified position in the expression.
      * @param first iterator starting the sequence of indices
      * @param last iterator ending the sequence of indices
-     * The number of indices in the squence should be equal to or greater
+     * The number of indices in the sequence should be equal to or greater
      * than the number of dimensions of the function.
      */
     template <class CT, class X>
