@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XGENERATOR_HPP
-#define XGENERATOR_HPP
+#ifndef XTENSOR_GENERATOR_HPP
+#define XTENSOR_GENERATOR_HPP
 
 #include <algorithm>
 #include <cstddef>
@@ -16,6 +16,8 @@
 #include <type_traits>
 #include <utility>
 
+#include "xtl/xsequence.hpp"
+
 #include "xexpression.hpp"
 #include "xiterable.hpp"
 #include "xstrides.hpp"
@@ -37,10 +39,6 @@ namespace xt
         using inner_shape_type = S;
         using const_stepper = xindexed_stepper<xgenerator<C, R, S>>;
         using stepper = const_stepper;
-        using const_iterator = xiterator<const_stepper, inner_shape_type*, DEFAULT_LAYOUT>;
-        using iterator = const_iterator;
-        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
-        using reverse_iterator = std::reverse_iterator<iterator>;
     };
 
     /**
@@ -56,9 +54,8 @@ namespace xt
      */
     template <class F, class R, class S>
     class xgenerator : public xexpression<xgenerator<F, R, S>>,
-                       public xexpression_const_iterable<xgenerator<F, R, S>>
+                       public xconst_iterable<xgenerator<F, R, S>>
     {
-
     public:
 
         using self_type = xgenerator<F, R, S>;
@@ -72,7 +69,7 @@ namespace xt
         using size_type = std::size_t;
         using difference_type = std::ptrdiff_t;
 
-        using iterable_base = xexpression_const_iterable<self_type>;
+        using iterable_base = xconst_iterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = inner_shape_type;
         using strides_type = S;
@@ -81,8 +78,8 @@ namespace xt
         using const_stepper = typename iterable_base::const_stepper;
 
         static constexpr layout_type static_layout = layout_type::any;
-        static constexpr bool contiguous_layout = true;
-        
+        static constexpr bool contiguous_layout = false;
+
         template <class Func>
         xgenerator(Func&& f, const S& shape) noexcept;
 
@@ -93,7 +90,12 @@ namespace xt
 
         template <class... Args>
         const_reference operator()(Args... args) const;
-        const_reference operator[](const xindex& index) const;
+        template <class... Args>
+        const_reference at(Args... args) const;
+        template <class OS>
+        disable_integral_t<OS, const_reference> operator[](const OS& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -108,7 +110,7 @@ namespace xt
         template <class O>
         const_stepper stepper_begin(const O& shape) const noexcept;
         template <class O>
-        const_stepper stepper_end(const O& shape) const noexcept;
+        const_stepper stepper_end(const O& shape, layout_type) const noexcept;
 
     private:
 
@@ -190,12 +192,39 @@ namespace xt
     template <class... Args>
     inline auto xgenerator<F, R, S>::operator()(Args... args) const -> const_reference
     {
-        XTENSOR_ASSERT(check_index(shape(), args...));
+        XTENSOR_TRY(check_index(shape(), args...));
         return m_f(args...);
     }
 
+    /**
+     * Returns a constant reference to the element at the specified position in the expression,
+     * after dimension and bounds checking.
+     * @param args a list of indices specifying the position in the function. Indices
+     * must be unsigned integers, the number of indices should be equal to the number of dimensions
+     * of the expression.
+     * @exception std::out_of_range if the number of argument is greater than the number of dimensions
+     * or if indices are out of bounds.
+     */
     template <class F, class R, class S>
-    inline auto xgenerator<F, R, S>::operator[](const xindex& index) const -> const_reference
+    template <class... Args>
+    inline auto xgenerator<F, R, S>::at(Args... args) const -> const_reference
+    {
+        check_access(shape(), args...);
+        return this->operator()(args...);
+    }
+
+    template <class F, class R, class S>
+    template <class OS>
+    inline auto xgenerator<F, R, S>::operator[](const OS& index) const
+        -> disable_integral_t<OS, const_reference>
+    {
+        return element(index.cbegin(), index.cend());
+    }
+
+    template <class F, class R, class S>
+    template <class I>
+    inline auto xgenerator<F, R, S>::operator[](std::initializer_list<I> index) const
+        -> const_reference
     {
         return element(index.begin(), index.end());
     }
@@ -210,14 +239,14 @@ namespace xt
      * Returns a constant reference to the element at the specified position in the function.
      * @param first iterator starting the sequence of indices
      * @param last iterator ending the sequence of indices
-     * The number of indices in the squence should be equal to or greater
+     * The number of indices in the sequence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class F, class R, class S>
     template <class It>
     inline auto xgenerator<F, R, S>::element(It first, It last) const -> const_reference
     {
-        XTENSOR_ASSERT(check_element_index(shape(), first, last));
+        XTENSOR_TRY(check_element_index(shape(), first, last));
         return m_f.element(first, last);
     }
     //@}
@@ -261,7 +290,7 @@ namespace xt
 
     template <class F, class R, class S>
     template <class O>
-    inline auto xgenerator<F, R, S>::stepper_end(const O& shape) const noexcept -> const_stepper
+    inline auto xgenerator<F, R, S>::stepper_end(const O& shape, layout_type) const noexcept -> const_stepper
     {
         size_type offset = shape.size() - dimension();
         return const_stepper(this, offset, true);
@@ -275,7 +304,7 @@ namespace xt
         {
             using shape_type = std::vector<std::size_t>;
             using type = xgenerator<Functor, typename Functor::value_type, shape_type>;
-            return type(std::forward<Functor>(f), forward_sequence<shape_type>(shape));
+            return type(std::forward<Functor>(f), xtl::forward_sequence<shape_type>(shape));
         }
 #else
         template <class Functor, class I, std::size_t L>
@@ -283,7 +312,7 @@ namespace xt
         {
             using shape_type = std::array<std::size_t, L>;
             using type = xgenerator<Functor, typename Functor::value_type, shape_type>;
-            return type(std::forward<Functor>(f), forward_sequence<shape_type>(shape));
+            return type(std::forward<Functor>(f), xtl::forward_sequence<shape_type>(shape));
         }
 #endif
 
