@@ -6,13 +6,15 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XSLICE_HPP
-#define XSLICE_HPP
+#ifndef XTENSOR_SLICE_HPP
+#define XTENSOR_SLICE_HPP
 
 #include <cstddef>
 #include <type_traits>
 #include <utility>
 
+#include "xtl/xtype_traits.hpp"
+
 #include "xutils.hpp"
 
 namespace xt
@@ -66,7 +68,7 @@ namespace xt
     using disable_xslice = typename std::enable_if<!is_xslice<E>::value, R>::type;
 
     template <class... E>
-    using has_xslice = or_<is_xslice<E>...>;
+    using has_xslice = xtl::disjunction<is_xslice<E>...>;
 
     /**********************
      * xrange declaration *
@@ -206,7 +208,7 @@ namespace xt
         inline std::enable_if_t<std::is_integral<MI>::value &&
                                 std::is_integral<MA>::value &&
                                 std::is_integral<STEP>::value, xstepped_range<int>>
-        get(std::size_t size) const
+        get(std::size_t /*size*/) const
         {
             return xstepped_range<int>(m_min, m_max, m_step);
         }
@@ -233,9 +235,9 @@ namespace xt
         inline std::enable_if_t<std::is_integral<MI>::value &&
                                 std::is_integral<MA>::value &&
                                !std::is_integral<STEP>::value, xrange<int>>
-        get(std::size_t size) const
+        get(std::size_t /*size*/) const
         {
-            return xrange<int>(m_min, m_max);
+            return xrange<int>(static_cast<int>(m_min), static_cast<int>(m_max));
         }
 
         template <class MI = A, class MA = B, class STEP = C>
@@ -255,7 +257,7 @@ namespace xt
                                 !std::is_integral<STEP>::value, xrange<std::size_t>>
         get(std::size_t size) const
         {
-            return xrange<std::size_t>((std::size_t)m_min, size);
+            return xrange<std::size_t>(std::size_t(m_min), size);
         }
 
         template <class MI = A, class MA = B, class STEP = C>
@@ -264,7 +266,7 @@ namespace xt
                                 !std::is_integral<STEP>::value, xrange<std::size_t>>
         get(std::size_t /*size*/) const
         {
-            return xrange<std::size_t>(0, (std::size_t)m_max);
+            return xrange<std::size_t>(0, std::size_t(m_max));
         }
 
         template <class MI = A, class MA = B, class STEP = C>
@@ -458,7 +460,7 @@ namespace xt
 
     template <class T>
     inline xstepped_range<T>::xstepped_range(size_type min_val, size_type max_val, size_type step) noexcept
-        : m_min(min_val), m_size((size_type)std::ceil(double(max_val - min_val) / double(step))), m_step(step)
+        : m_min(min_val), m_size(size_type(std::ceil(double(max_val - min_val) / double(step)))), m_step(step)
     {
     }
 
